<?php

namespace catchAdmin\tag_history\controller;

use catchAdmin\report\model\Report;
use catcher\base\CatchRequest as Request;
use catcher\CatchResponse;
use catcher\base\CatchController;
use catchAdmin\tag_history\model\TagHistory as tagHistoryModel;
use catcher\CatchAdmin;
use catcher\Code;
use think\facade\Db;

class TagHistory extends CatchController
{
    protected $tagHistoryModel;

    public function __construct(TagHistoryModel $tagHistoryModel)
    {
        $this->tagHistoryModel = $tagHistoryModel;
    }

    /**
     * 列表
     * @time 2022年08月12日 11:31
     * @param Request $request 
     */
    public function index(Request $request): \think\Response
    {

        $data = $request->get();
        $time =  isset($data['time']) ? $data['time'] : '';
        $mac = isset($data['mac']) ? $data['mac'] : '';
        $tage = isset($data['tage']) ? $data['tage'] : '';
        $endtime = isset($data['endtime']) ? $data['endtime'] : "";
        $meter = isset($data['meter']) ? $data['meter'] : "";
        $where = [];
        $create_id = $request->user()->id;

        $bool = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 3)->find();
        $shifouRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 4)->find();
        $ccccRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 5)->find();
        //
        $six = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 6)->find();

        if (!empty($time)) {
            $where[] = ['addTime', '>=', $time];
        } else {
            $where[] = ['addTime', '>=', time()];
        }

        if (!empty($mac)) {
            $where[] = ['mac', 'in',  implode(",", $mac)];
        }
        if (!empty($tage)) {
            $where[] = ['lable', 'in',  implode(",", $tage)];
        }
        if ($endtime == 1) {
            //生成报告
            $end = time();
            if (empty($mac)) {
                $mac = $this->tagHistoryModel->group('mac')->column('mac');
            }
            if (empty($tage)) {
                $tage = $this->tagHistoryModel->group('lable')->column('lable');
            }
            $tagList =  $this->tagHistoryModel->where($where)->where('addTime', '<=', $end)->select();
            $content = [
                'mac' => json_encode($mac),
                'lable' => json_encode($tage),
                'meter' => $meter,
                'startTime' => $time,
                'endTime' => $end,
                'data' => json_encode($tagList)
            ];
            $report = new Report();
            $report->save($content);
            return CatchResponse::success('生成报告成功');
        }

        if ($bool) {
            $where[] = ['mac', 'in', '30B5F1013899'];
            $where[] = ['lable', 'in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306"];
        } elseif ($shifouRole) {
            $where[] = ['mac', 'in', '30B5F10138EB'];
            $where[] = ['lable', 'in', "30b5f6000006,30b5f6000007,30b5f6000008,0000f6000006,0000f6000007,0000f6000008"];
        } elseif ($ccccRole) {
            $where[] = ['mac', 'in', '30B5F10138ED,30B5F1012F7A'];
            $where[] = ['lable', 'in', "000000880002,000000880003,000000880004"];
        } elseif ($six) {

            $where[] = ['mac', 'in', '30B5F1013801'];
            $where[] = ['lable', 'in', "0000e2b7a905,0000e2b7a2c5,0000ea511abe,0000e2b6d355"];
        } else {

            $where[] = ['mac', 'not in', '30B5F1013899'];
            $where[] = ['mac', 'not in', '30B5F10138EB'];
            $where[] = ['mac', 'not in', '30B5F10138ED,30B5F1012F7A'];
            $where[] = ['mac', 'not in', '30B5F1013801'];
            $where[] = ['lable', 'not in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306"];
            $where[] = ['lable', 'not in', "30b5f6000006,30b5f6000007,30b5f6000008,0000f6000006,0000f6000007,0000f6000008"];
            $where[] = ['lable', 'not in', "000000880002,000000880003,000000880004"];
            $where[] = ['lable', 'not in', "0000e2b7a905,0000e2b7a2c5,0000ea511abe,0000e2b6d355"];
        }




        $list = $this->tagHistoryModel->group('mac,lable')->where($where)
            ->field('mac,lable,max(addTime) as addTime,format(AVG(rssi),2) as ave_rssi,count(*) as num')->select()->toArray();

        foreach ($list as $key => $value) {
            $wheres = [];
            $da = null;
            $wheres[] = ['mac', '=', $value['mac']];
            $wheres[] = ['lable', '=', $value['lable']];
            $wheres[] = ['addTime', '=', $value['addTime']];
            $da = $this->tagHistoryModel->where($wheres)->find();
            $list[$key]['rssi'] = $da['rssi'];
            $list[$key]['move'] = $da['move'];
            $list[$key]['step'] = $da['step'];
            $list[$key]['temperature'] = $da['temperature'];
            $list[$key]['addTime'] = date('Y-m-d H:i:s', $value['addTime']);
            $list[$key]['time'] = date('Y-m-d H:i:s', $da['time']);
        }
        return CatchResponse::success($list);
    }

    /**
     * 保存信息
     * @time 2022年08月12日 11:31
     * @param Request $request 
     */
    public function save(Request $request): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->storeBy($request->post()));
    }

    /**
     * 读取
     * @time 2022年08月12日 11:31
     * @param $id 
     */
    public function read($id): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->findBy($id));
    }

    /**
     * 更新
     * @time 2022年08月12日 11:31
     * @param Request $request 
     * @param $id
     */
    public function update(Request $request, $id): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->updateBy($id, $request->post()));
    }
    /**
     * 删除
     * @time 2022年08月12日 11:31
     * @param $id
     */
    public function delete($id): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->deleteBy($id));
    }

    /**
     * @Descripttion: 获取设备
     * @name: likang
     * @return {*}
     */
    public function getStations(Request $request)
    {
        $create_id = $request->user()->id;
        $where = [];
        $bool = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 3)->find();
        $shifouRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 4)->find();
        $six = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 6)->find();
        $ccccRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 5)->find();

        if ($bool) {
            $where[] = ['mac', 'in', '30B5F1013899'];
        } else if ($shifouRole) {
            $where[] = ['mac', 'in', '30B5F10138EB'];
        } elseif ($ccccRole) {
            $where[] = ['mac', 'in', '30B5F10138ED,30B5F1012F7A'];
        } elseif ($six) {
            $where[] = ['mac', 'in', '30B5F1013801'];
        } else {
            $where[] = ['mac', 'not in', ['30B5F1013899', '30B5F10138EB', '30B5F1013801', '30B5F10138ED']];
        }
        $list = $this->tagHistoryModel->group('mac')->where($where)->column('mac');
        return CatchResponse::success($list);
    }
    /**
     * @Descripttion: 获取标签
     * @name: likang
     * @return {*}
     */
    public function getTages(Request $request)
    {
        $create_id = $request->user()->id;
        $where = [];
        $bool = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 3)->find();
        $shifouRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 4)->find();
        $six = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 6)->find();
        //牛耳朵
        $ccccRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 5)->find();

        if ($bool) {
            $where[] = ['lable', 'in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306"];
        } elseif ($shifouRole) {
            $where[] = ['lable', 'in', "30b5f6000006,30b5f6000007,30b5f6000008,0000f6000006,0000f6000007,0000f6000008"];
        } elseif ($ccccRole) {
            $where[] = ['lable', 'in', "000000880002,000000880003,000000880004"];
        } elseif ($six) {
            $where[] = ['lable', 'in', "0000e2b7a905,0000e2b7a2c5,0000ea511abe,0000e2b6d355"];
        } else {
            $where[] = ['lable', 'not in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306"];
            $where[] = ['lable', 'not in', "30b5f6000006,30b5f6000007,30b5f6000008,0000f6000006,0000f6000007,0000f6000008"];
            $where[] = ['lable', 'not in', "0000e2b7a905,0000e2b7a2c5,0000ea511abe,0000e2b6d355"];
            $where[] = ['lable', 'not in', "000000880002,000000880003,000000880004"];
        }
        $list = $this->tagHistoryModel->group('lable')->where($where)->column('lable');
        return CatchResponse::success($list);
    }
    /**
     * @Descripttion: 获取时间
     * @name: likang
     * @return {*}
     */
    public function getTime()
    {

        $time = time();
        return CatchResponse::success($time);
    }
    /**
     * @Descripttion: 根据标签和设备获取数据和心跳
     * @name: likang
     * @param {Request} $request
     * @return {*}
     */
    public function getRecordByTag(Request $request)
    {
        $lable = $request->get('lable');
        $mac = $request->get('mac');
        $limit = 10;
        $page = 1;
        if ($request->get('page')) {
            $page = $request->get('page');
        }
        if ($request->get('limit')) {
            $limit = $request->get('limit');
        }
        $where = [
            ['lable', '=', $lable],
            ['mac', '=', $mac]
        ];
        $list = $this->tagHistoryModel->where($where)->order('addTime desc')->page($page)->limit($limit)->select();
        foreach ($list as $key => $value) {
            $list[$key]['time'] = date('Y-m-d H:i:s', $value['time']);
            $list[$key]['addTime'] = date('Y-m-d H:i:s', $value['addTime']);
        }
        return json([
            'code'    => Code::SUCCESS,
            'message' => 'success',
            'count'   => $this->tagHistoryModel->where($where)->count(),
            'current' => $page,
            'limit'   => $limit,
            'data'    => $list,
        ]);
    }
    //随机生成温度
    public function randomTemperature()
    {

        $resout = 38.5;
        return $resout;
    }
    //生成随机步数
    public function randomStep()
    {
        $intage = 10;
        $resout = $intage;
        return $resout;
    }
    /**
     * 折现图
     */
    public function lineChart(Request $request)
    {
        $data = $request->get();
        $start_time = $data['start_time'];

        if (empty($start_time)) {
            $start_time = strtotime(date('Y-m-d'));
        }

        $end_time = null;
        if (!$end_time) {
            $end_time = time();
        }

        $where = [];
        $where[] = [$start_time, '>=', 'addTime'];
        $where[] = [$end_time, '=<', 'addTime'];
        $where[] = ['mac', 'in', '30B5F10138ED'];
        $where[] = ['lable', 'in', "000000880002,000000880003"];

        $list = $this->tagHistoryModel->where($where)->order('addTime desc')->select()->toArray();

        $time = [];
        $lable1 = [];
        $lable2 = [];
        foreach ($list as $item) {
            $time[] = $item['addTime'];
            if ($item['lable'] == '000000880002') {
                $lable1[$item['addTime']] = $item['temperature'];
                if (!$lable2[$item['addTime']]) {
                    $lable2[$item['addTime']] = null;
                }
            } else if ($item['lable'] == '000000880003') {
                $lable2[] = $item['temperature'];
                if (!$lable1[$item['addTime']]) {
                    $lable1[$item['addTime']] = null;
                }
            }
        }
        $renData = [];
        $renData = [
            'xAxis' => array_unique($time),
            'legend' => ['000000880002', '000000880003'],
            'series' => [
                [
                    'name' => '000000880002',
                    'type' => 'line',
                    'stack' => 'Total',
                    'data' => array_values($lable1)
                ],
                [
                    'name' => '000000880003',
                    'type' => 'line',
                    'stack' => 'Total',
                    'data' => array_values($lable2)
                ]
            ]

        ];
        return CatchResponse::success($renData);
    }
}
